// src/index.ts
import { createRequire } from "module";
import { fromZodError } from "zod-validation-error";
import {
  DEFAULT_CONFIG,
  DEFAULT_LOCALE,
  DEFAULT_LOCALES,
  MARKDOWN_EXTENSION_REGEX,
  MARKDOWN_EXTENSIONS
} from "./constants.mjs";
import { nextraConfigSchema } from "./schemas.mjs";
import { logger } from "./utils.mjs";
import { NextraPlugin, NextraSearchPlugin } from "./webpack-plugins/index.mjs";
var DEFAULT_EXTENSIONS = ["js", "jsx", "ts", "tsx"];
var require2 = createRequire(import.meta.url);
var nextra = (nextraConfig) => {
  try {
    nextraConfigSchema.parse(nextraConfig);
  } catch (error) {
    logger.error("Error validating nextraConfig");
    throw fromZodError(error);
  }
  return function withNextra(nextConfig = {}) {
    const hasI18n = !!nextConfig.i18n?.locales;
    if (hasI18n) {
      logger.info(
        "You have Next.js i18n enabled, read here https://nextjs.org/docs/advanced-features/i18n-routing for the docs."
      );
      logger.warn(
        "When i18n enabled, Nextra set nextConfig.i18n = undefined, use `useRouter` from `nextra/hooks` if you need `locale` or `defaultLocale` values."
      );
    }
    const locales = nextConfig.i18n?.locales || DEFAULT_LOCALES;
    const rewrites = async () => {
      const rules = [{ source: "/:path*/_meta", destination: "/404" }];
      if (nextConfig.rewrites) {
        const originalRewrites = await nextConfig.rewrites();
        if (Array.isArray(originalRewrites)) {
          return [...originalRewrites, ...rules];
        }
        return {
          ...originalRewrites,
          beforeFiles: [...originalRewrites.beforeFiles || [], ...rules]
        };
      }
      return rules;
    };
    const loaderOptions = {
      ...DEFAULT_CONFIG,
      ...nextraConfig,
      locales
    };
    if (!nextraConfig.theme) {
      throw new Error("No Nextra theme found!");
    }
    return {
      ...nextConfig,
      ...nextConfig.output !== "export" && { rewrites },
      env: {
        ...nextConfig.env,
        ...hasI18n && {
          NEXTRA_DEFAULT_LOCALE: nextConfig.i18n?.defaultLocale || DEFAULT_LOCALE
        },
        NEXTRA_SEARCH: String(!!loaderOptions.search)
      },
      ...hasI18n && { i18n: void 0 },
      pageExtensions: [
        ...nextConfig.pageExtensions || DEFAULT_EXTENSIONS,
        ...MARKDOWN_EXTENSIONS
      ],
      webpack(config, options) {
        if (options.nextRuntime !== "edge" && options.isServer) {
          config.plugins || (config.plugins = []);
          config.plugins.push(new NextraPlugin({ locales }));
          if (loaderOptions.search) {
            config.plugins.push(new NextraSearchPlugin());
          }
        }
        const defaultESMAppPath = require2.resolve("next/dist/esm/pages/_app.js");
        const defaultCJSAppPath = require2.resolve("next/dist/pages/_app.js");
        config.resolve.alias = {
          ...config.resolve.alias,
          // Resolves ESM _app file instead cjs, so we could import theme.config via `import` statement
          [defaultCJSAppPath]: defaultESMAppPath
        };
        config.module.rules.push(
          {
            // Match Markdown imports from non-pages. These imports have an
            // issuer, which can be anything as long as it's not empty.
            // When the issuer is null, it means that it can be imported via a
            // runtime import call such as `import('...')`.
            test: MARKDOWN_EXTENSION_REGEX,
            issuer: (request) => !!request || request === null,
            use: [
              options.defaultLoaders.babel,
              {
                loader: "nextra/loader",
                options: loaderOptions
              }
            ]
          },
          {
            // Match pages (imports without an issuer request).
            test: MARKDOWN_EXTENSION_REGEX,
            issuer: (request) => request === "",
            use: [
              options.defaultLoaders.babel,
              {
                loader: "nextra/loader",
                options: {
                  ...loaderOptions,
                  isPageImport: true
                }
              }
            ]
          },
          {
            // Match dynamic meta files inside pages.
            test: /_meta\.js$/,
            issuer: (request) => !request,
            use: [options.defaultLoaders.babel, { loader: "nextra/loader" }]
          },
          {
            test: /pages\/_app\./,
            issuer: (request) => !request,
            use: [
              options.defaultLoaders.babel,
              {
                loader: "nextra/loader",
                options: loaderOptions
              }
            ]
          }
        );
        return nextConfig.webpack?.(config, options) || config;
      }
    };
  };
};
var src_default = nextra;
export {
  src_default as default
};
