// src/plugin.ts
import path from "path";
import { promisify } from "util";
import fs from "graceful-fs";
import grayMatter from "gray-matter";
import pLimit from "p-limit";
import {
  CWD,
  DYNAMIC_META_FILENAME,
  MARKDOWN_EXTENSION_REGEX,
  META_FILENAME
} from "./constants.mjs";
import {
  isSerializable,
  logger,
  normalizePageRoute,
  sortPages,
  truthy
} from "./utils.mjs";
var readdir = promisify(fs.readdir);
var readFile = promisify(fs.readFile);
var realpath = promisify(fs.realpath);
var stat = promisify(fs.stat);
async function collectMdx(filePath, route = "") {
  const { name } = path.parse(filePath);
  const content = await readFile(filePath, "utf8");
  const { data } = grayMatter(content);
  return {
    kind: "MdxPage",
    name,
    route,
    ...Object.keys(data).length && { frontMatter: data }
  };
}
var limit = pLimit(20);
async function collectFiles({
  dir,
  route = "/",
  fileMap = /* @__PURE__ */ Object.create(null),
  isFollowingSymlink = false
}) {
  var _a;
  const files = await readdir(dir, { withFileTypes: true });
  const promises = files.map(async (f) => {
    const filePath = path.join(dir, f.name);
    let isDirectory = f.isDirectory();
    const isSymlinked = isFollowingSymlink || f.isSymbolicLink();
    let symlinkSource;
    if (isSymlinked) {
      symlinkSource = await realpath(filePath);
      const stats = await stat(filePath);
      if (stats.isDirectory()) {
        isDirectory = true;
      }
    }
    const { name, ext } = path.parse(filePath);
    if (name.startsWith("["))
      return;
    const fileRoute = normalizePageRoute(route, name);
    if (isDirectory) {
      if (fileRoute === "/api")
        return;
      const { items: items2 } = await collectFiles({
        dir: filePath,
        route: fileRoute,
        fileMap,
        isFollowingSymlink: isSymlinked
      });
      if (!items2.length)
        return;
      return {
        kind: "Folder",
        name: f.name,
        route: fileRoute,
        children: items2
      };
    }
    return limit(async () => {
      if (MARKDOWN_EXTENSION_REGEX.test(ext)) {
        if (fileRoute === "/_app")
          return;
        const fp = filePath;
        fileMap[fp] = await collectMdx(fp, fileRoute);
        if (symlinkSource) {
          fileMap[symlinkSource] = { ...fileMap[fp] };
        }
        return fileMap[fp];
      }
      const fileName = name + ext;
      try {
        if (fileName === META_FILENAME) {
          const fp = filePath;
          const content = await readFile(fp, "utf8");
          fileMap[fp] = {
            kind: "Meta",
            data: JSON.parse(content)
          };
          return fileMap[fp];
        }
        if (fileName === DYNAMIC_META_FILENAME) {
          const importPath = `${filePath}?d=${Date.now()}`;
          const metaMod = await import(importPath);
          const meta = metaMod.default;
          const fp = filePath.replace(/\.js$/, ".json");
          if (typeof meta === "function") {
            fileMap[fp] = {
              kind: "Meta",
              __nextra_src: filePath,
              data: {}
            };
          } else if (meta && typeof meta === "object" && isSerializable(meta)) {
            fileMap[fp] = {
              kind: "Meta",
              // we spread object because default title could be incorrectly set when _meta.json/js
              // is imported/exported by another _meta.js
              data: { ...meta }
            };
          } else {
            logger.error(
              `"${fileName}" is not a valid meta file. The default export is required to be a serializable object or a function. Please check the following file:`,
              path.relative(CWD, filePath)
            );
          }
          return fileMap[fp];
        }
      } catch (error) {
        const relPath = path.relative(CWD, filePath);
        logger.error(
          `Error loading ${relPath}
${error.name}: ${error.message}`
        );
      }
      if (fileName === "meta.json") {
        logger.warn(
          '"meta.json" was renamed to "_meta.json". Rename the following file:',
          path.relative(CWD, filePath)
        );
      } else if (/_meta\.(jsx|ts|tsx|cjs|mjs)$/.test(fileName)) {
        logger.error(
          `"${fileName}" is not currently supported, please rename the following file to "${DYNAMIC_META_FILENAME}":`,
          path.relative(CWD, filePath)
        );
      }
    });
  });
  const items = (await Promise.all(promises)).filter(truthy);
  const mdxPages = [];
  let metaFile;
  for (const item of items) {
    if (item.kind === "MdxPage") {
      mdxPages.push(item);
    } else if (item.kind === "Meta") {
      metaFile = item;
    }
  }
  const defaultMeta = sortPages(mdxPages);
  if (!metaFile && mdxPages.length > 0) {
    metaFile = {
      kind: "Meta",
      data: Object.fromEntries(defaultMeta)
    };
    items.push(metaFile);
  }
  if (metaFile) {
    for (const [key, capitalizedTitle] of defaultMeta) {
      (_a = metaFile.data)[key] || (_a[key] = capitalizedTitle);
      const metaItem = metaFile.data[key];
      if (typeof metaItem === "object") {
        metaItem.title || (metaItem.title = capitalizedTitle);
      }
    }
  }
  return { items, fileMap };
}
export {
  collectFiles,
  collectMdx
};
